unit Oblong.KelvinEditor;

interface

uses
  System.Drawing, System.Collections, System.ComponentModel,
  System.Windows.Forms, System.Drawing.Design,
  System.Windows.Forms.Design;

type
  KelvinList = class(System.Windows.Forms.UserControl)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    Components: System.ComponentModel.Container;
    ValuesList: System.Windows.Forms.ListBox;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
    procedure ValuesList_DrawItem(sender: System.Object; e: System.Windows.Forms.DrawItemEventArgs);
  {$ENDREGION}
  strict protected
    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    procedure Dispose(Disposing: Boolean); override;
  private
    { Private Declarations }
  public
    constructor Create;
    property List: ListBox read ValuesList;
  end;

  [assembly: RuntimeRequiredAttribute(TypeOf(KelvinList))]

  KelvinEditor = class(UITypeEditor)
  protected
    EdSvc: IWindowsFormsEditorService;
    procedure ListSelectedIndexChanged(sender: System.Object; e: System.EventArgs);
  public
    function EditValue(context: ITypeDescriptorContext;
      provider: IServiceProvider; value: TObject): TObject; override;
    function GetEditStyle(context: ITypeDescriptorContext): UITypeEditorEditStyle; override;
    function GetPaintValueSupported(context: ITypeDescriptorContext): Boolean;
      override;
    procedure PaintValue(e: PaintValueEventArgs); override;
  end;

implementation

uses
  System.Globalization;

{$AUTOBOX ON}

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support -- do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure KelvinList.InitializeComponent;
type
  TArrayOfSystem_Object = array of System.Object;
begin
  Self.ValuesList := System.Windows.Forms.ListBox.Create;
  Self.SuspendLayout;
  //
  // ValuesList
  //
  Self.ValuesList.Anchor := (System.Windows.Forms.AnchorStyles((((System.Windows.Forms.AnchorStyles.Top
    or System.Windows.Forms.AnchorStyles.Bottom) or System.Windows.Forms.AnchorStyles.Left)
    or System.Windows.Forms.AnchorStyles.Right)));
  Self.ValuesList.DrawMode := System.Windows.Forms.DrawMode.OwnerDrawFixed;
  Self.ValuesList.Items.AddRange(TArrayOfSystem_Object.Create('Absolute zero',
          'Freezing', 'Room temperature', 'Boiling'));
  Self.ValuesList.Location := System.Drawing.Point.Create(1, 0);
  Self.ValuesList.Name := 'ValuesList';
  Self.ValuesList.Size := System.Drawing.Size.Create(120, 56);
  Self.ValuesList.TabIndex := 0;
  Include(Self.ValuesList.DrawItem, Self.ValuesList_DrawItem);
  //
  // KelvinList
  //
  Self.Controls.Add(Self.ValuesList);
  Self.Name := 'KelvinList';
  Self.Size := System.Drawing.Size.Create(120, 56);
  Self.ResumeLayout(False);
end;
{$ENDREGION}

constructor KelvinList.Create;
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent;
  //
  // TODO: Add any constructor code after InitializeComponent call
  //
end;

procedure KelvinList.ValuesList_DrawItem(sender: System.Object;
  e: System.Windows.Forms.DrawItemEventArgs);
var
  e2: DrawItemEventArgs;
  BkCol: Color;
begin
  case e.Index of
    0: BkCol := Color.Navy;
    1: BkCol := Color.Blue;
    2: BkCol := Color.Lime;
    3: BkCol := Color.Red;
  end;
  e2 := DrawItemEventArgs.Create(
    e.Graphics, e.Font, e.Bounds, e.Index, e.State, e.ForeColor, BkCol);
  e2.DrawBackground;
  e2.Graphics.DrawString(ValuesList.Items[e2.Index].ToString, e2.Font,
    Brushes.White, e2.Bounds);
  e2.DrawFocusRectangle;
end;

procedure KelvinList.Dispose(Disposing: Boolean);
begin
  if Disposing then
  begin
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

{ KelvinEditor }

function KelvinEditor.EditValue(context: ITypeDescriptorContext;
  provider: IServiceProvider; value: TObject): TObject;
var
  Lst: KelvinList;
begin
  EdSvc := IWindowsFormsEditorService(provider.GetService(TypeOf(IWindowsFormsEditorService)));
  if Assigned(EdSvc) then
  begin
    Lst := KelvinList.Create;
    Include(Lst.List.SelectedIndexChanged, ListSelectedIndexChanged);
    EdSvc.DropDownControl(Lst);
    case Lst.List.SelectedIndex of
      0: Result := 0;
      1: Result := 273.15;
      2: Result := 297.15;
    else
         Result := 373.15;
    end;
  end
  else
    Result := value
end;

function KelvinEditor.GetEditStyle(
  context: ITypeDescriptorContext): UITypeEditorEditStyle;
begin
  Result := UITypeEditorEditStyle.DropDown
end;

function KelvinEditor.GetPaintValueSupported(
  context: ITypeDescriptorContext): Boolean;
begin
  Result := True
end;

procedure KelvinEditor.ListSelectedIndexChanged(sender: TObject;
  e: System.EventArgs);
begin
  EdSvc.CloseDropDown;
end;

procedure KelvinEditor.PaintValue(e: PaintValueEventArgs);
var
  MyBrush: Brush;
  Val: Double;
begin
  Val := Convert.ToDouble(e.Value);
  if Val < 200 then
    MyBrush := Brushes.Navy
  else if (Val >= 200) and (Val <= 278.15) then
    MyBrush := Brushes.Blue
  else if (Val > 278.15) and (Val < 373) then
    MyBrush := Brushes.Lime
  else
    MyBrush := Brushes.Red;
  e.Graphics.FillRectangle(MyBrush, e.Bounds);
end;

end.
