unit eBob42.ShadingLabel;
interface
uses
  SysUtils, Classes, Controls, StdCtrls;

type
  TLabelStyle = (lsLowered, lsNormal, lsRaised);

type
  TShadingLabel = class(TLabel)
  private
    FLabelStyle: TLabelStyle;
    FOffset: Integer;
    procedure SetLabelStyle(const Value: TLabelStyle);
    procedure SetOffset(const Value: Integer);
    { Private declarations }
  protected
    procedure Paint; override;
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    { Public declarations }
  published
    { Published declarations }
    property LabelStyle: TLabelStyle read FLabelStyle write SetLabelStyle default lsLowered;
    property Offset: Integer read FOffset write SetOffset default 1;
  end;

implementation
uses
{$IFDEF CLR}
  Borland.Vcl.Types,
{$ENDIF}
  Windows, Graphics;

{ TBLabel }

constructor TShadingLabel.Create(AOwner: TComponent);
begin
  inherited;
  FOffset := 1;
//FLabelStyle := lsLowered;
  Transparent := True
end;

procedure TShadingLabel.SetLabelStyle(const Value: TLabelStyle);
begin
  if FLabelStyle <> Value then
  begin
    FLabelStyle := Value;
    Invalidate
  end
end;

procedure TShadingLabel.SetOffset(const Value: Integer);
begin
  if FOffset <> Value then
  begin
    FOffset := Value;
    Invalidate
  end
end;

procedure TShadingLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Len: Integer; // Caption Length
  CalcOffset: Integer;
  OldColor: TColor;
  TempRect: TRect;
begin
  Len := Length(Caption);
  CalcOffset := (Ord(FLabelStyle)-1) * FOffset;
  OldColor := Font.Color;
  if CalcOffset <> 0 then
  try
    Canvas.Brush.Style := bsClear;
    Canvas.Font := Self.Font;
    Canvas.Font.Color := clWhite; // white light
    TempRect := Rect(ClientRect.Left - CalcOffset,
                     ClientRect.Top - CalcOffset,
                     ClientRect.Right - CalcOffset,
                     ClientRect.Bottom - CalcOffset);
    {$IFDEF WIN32}
    DrawText(Canvas.Handle, PChar(Caption), Len, TempRect,
      DT_EXPANDTABS or DT_WORDBREAK or Alignments[Alignment]);
    {$ELSE}
    DrawText(Canvas.Handle, Caption, Len, TempRect,
      DT_EXPANDTABS or DT_WORDBREAK or Alignments[Alignment]);
    {$ENDIF}
    Canvas.Font.Color := clBlack; // black shadow
    TempRect := Rect(ClientRect.Left + CalcOffset,
                     ClientRect.Top + CalcOffset,
                     ClientRect.Right + CalcOffset,
                     ClientRect.Bottom + CalcOffset);
    {$IFDEF WIN32}
    DrawText(Canvas.Handle, PChar(Caption), Len, TempRect,
      DT_EXPANDTABS or DT_WORDBREAK or Alignments[Alignment])
    {$ELSE}
    DrawText(Canvas.Handle, Caption, Len, TempRect,
      DT_EXPANDTABS or DT_WORDBREAK or Alignments[Alignment])
    {$ENDIF}
  finally
    Canvas.Font.Color := OldColor
  end;
  inherited Paint // paint original Caption
end;

end.
